#version 430 core


in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

out vec4 color;


uniform float brightness;
uniform vec3 camera_world;
uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
uniform sampler2D diffuseTexture;


uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL;

vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation,float brightness) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	return (diffuseF * diffuseC * max(0, dot(n, l)) + specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att*brightness; 
}



void main() {	
	vec3 n = normalize(vert.normal_world);
	vec3 dirLightDirection = normalize(-dirL.direction);
	color.rgb = vec4(1, 0.5, 0, 1).rgb; // ambient	
	float intensity = dot(n,dirLightDirection);
	vec3 celColor;
	if(intensity > 0.90) celColor = color.rgb*dirL.color;
	else if(intensity > 0.5) celColor = color.rgb*dirL.color*0.35;
	else celColor = vec3(0.0);
	
	color.rgb *= celColor;
	
		
	color.rgb += phong(n, -dirL.direction ,normalize(camera_world-vert.position_world) ,dirL.color*vec4(1, 0.5, 0, 1).rgb ,materialCoefficients.y ,dirL.color,
		     materialCoefficients.z ,specularAlpha, false, vec3(0),brightness);
	color.rgb += phong(n, pointL.position - vert.position_world, normalize(camera_world-vert.position_world), pointL.color*vec4(1, 0.5, 0, 1).rgb, materialCoefficients.y,
		     pointL.color, materialCoefficients.z, specularAlpha, true, pointL.attenuation,brightness);
}
